"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _drained, _done;
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkerPoolAwsSdk = void 0;
const path_1 = __importDefault(require("path"));
const piscina_1 = __importDefault(require("piscina"));
const serialize_error_1 = require("serialize-error");
class WorkerPool extends piscina_1.default {
    constructor(poolOptions) {
        super(poolOptions);
        _drained.set(this, true);
        _done.set(this, false);
        this.restart();
        this.on('drain', () => {
            __classPrivateFieldSet(this, _drained, true);
        });
    }
    get done() {
        return __classPrivateFieldGet(this, _done);
    }
    get drained() {
        return __classPrivateFieldGet(this, _drained);
    }
    end() {
        __classPrivateFieldSet(this, _done, true);
    }
    begin() {
        __classPrivateFieldSet(this, _drained, false);
    }
    restart() {
        __classPrivateFieldSet(this, _drained, true);
        __classPrivateFieldSet(this, _done, false);
    }
    get isFinished() {
        return !this.queueSize && this.drained && this.done;
    }
    async shutdown(doDestroy = true) {
        this.end();
        await new Promise((resolve, reject) => {
            if (this.isFinished) {
                resolve(null);
            }
            else {
                this.once('error', reject);
                this.once('drain', resolve);
            }
        });
        if (doDestroy) {
            await this.destroy();
        }
        return true;
    }
}
_drained = new WeakMap(), _done = new WeakMap();
class WorkerPoolAwsSdk extends WorkerPool {
    constructor(poolOptions) {
        super({
            filename: path_1.default.resolve(__dirname, '../dist/workers/aws-sdk.js'),
            idleTimeout: 25000,
            ...poolOptions,
        });
    }
    async runAwsTask(params) {
        if (this.done) {
            throw Error('Not allowed to make an API call after the worker pool has been flagged as done.');
        }
        this.begin();
        try {
            const taskInput = JSON.parse(JSON.stringify(params));
            const result = await this.runTask(taskInput);
            return result;
        }
        catch (err) {
            if (typeof err === 'string') {
                throw serialize_error_1.deserializeError(err);
            }
            this.emit('drain');
            throw err;
        }
    }
}
exports.WorkerPoolAwsSdk = WorkerPoolAwsSdk;
