import { AWSError } from 'aws-sdk';
import { PromiseResult } from 'aws-sdk/lib/request';
import { Service, ServiceConfigurationOptions } from 'aws-sdk/lib/service';
export declare type Constructor<T = unknown> = new (...args: any[]) => T;
export declare type InstanceProperties<T extends Service = Service, C extends Constructor<T> = Constructor<T>> = keyof InstanceType<C>;
export declare type ServiceProperties<S extends Service = Service, C extends Constructor<S> = Constructor<S>> = Exclude<InstanceProperties<S, C>, InstanceProperties<Service, Constructor<Service>>>;
export declare type OverloadedArguments<T> = T extends {
    (...args: any[]): any;
    (params: infer P, callback: any): any;
    (callback: any): any;
} ? P : T extends {
    (params: infer P, callback: any): any;
    (callback: any): any;
} ? P : T extends (params: infer P, callback: any) => any ? P : any;
export declare type Result<T> = T extends (...args: any) => infer R ? R : any;
export declare type Input<T> = T extends (...args: infer P) => any ? P : never;
export declare type ServiceOptions<S extends Service = Service> = ConstructorParameters<Constructor<S>>[0];
export declare type ServiceOperation<S extends Service = Service, C extends Constructor<S> = Constructor<S>, O extends ServiceProperties<S, C> = ServiceProperties<S, C>, E extends Error = AWSError> = InstanceType<C>[O] & {
    promise(): Promise<PromiseResult<any, E>>;
};
export declare type InferredResult<S extends Service = Service, C extends Constructor<S> = Constructor<S>, O extends ServiceProperties<S, C> = ServiceProperties<S, C>, E extends Error = AWSError, N extends ServiceOperation<S, C, O, E> = ServiceOperation<S, C, O, E>> = Input<Input<Result<Result<N>['promise']>['then']>[0]>[0];
export interface ClientApiOptions<S extends Service = Service, C extends Constructor<S> = Constructor<S>, O extends ServiceProperties<S, C> = ServiceProperties<S, C>, E extends Error = AWSError, N extends ServiceOperation<S, C, O, E> = ServiceOperation<S, C, O, E>> {
    name: string;
    options: ServiceOptions<S> | ServiceConfigurationOptions;
    operation: O | string;
    input?: OverloadedArguments<N>;
    headers?: Record<string, string>;
}
export default function clientApi<S extends Service = Service, C extends Constructor<S> = Constructor<S>, O extends ServiceProperties<S, C> = ServiceProperties<S, C>, E extends Error = AWSError, N extends ServiceOperation<S, C, O, E> = ServiceOperation<S, C, O, E>>(params: ClientApiOptions<S, C, O, E, N>): Promise<InferredResult<S, C, O, E, N>>;
